/*                                                                          
  Ϳ
   VGAregs.h                                                              
                                                                          
   Librera de acceso (lectura/escritura) a los registros de las tarjetas 
   de vdeo, teniendo definidos los controladores de las VGA estndar.    
                                                                          
   Para incluirla en cualquier programa C basta con usar la orden         
   #include "VGAREG.H"                    al principio del mismo.         
                                                                          
   El uso de la funcin outport() requiere el header <dos.h> .            
                                                                          
   Cdigo y texto:  [NoP] de Compiler SoftWare.                     
  1997 C.S.
*/
  
/* Controladores comentados en el artculo */
#define ATTRC      0x3c0
#define VGASEQ     0x3c4
#define GRAPHC     0x3ce
#define CRTC       0x3d4
  
/* Otros registros o controladores en la tarjeta */
#define STATUS_P   0x3da
#define MISCREG_R  0x3c2
#define MISCREG_W  0x3c3

/*  Funciones implementadas:  */
/**/
char ReadVGAREG( int, char );
void WriteVGAREG( int, char, char );
void UnProtectCRTC( void );

void GRAPHCwrite( char, char );
char GRAPHCread( char );
void ATTRCwrite( char, char );
char ATTRCread( char );
void CRTCwrite( char, char );
char CRTCread( char );
void VGASEQwrite( char, char );
char VGASEQread( char );
/**/


/*========================================================================
   valor = ReadVGAREG( controlador, index );

   Lee el registro especificado (controlador->index) y devuelve el valor
   que contiene. Controla los posibles casos comentados en la revista.
 ========================================================================*/
char ReadVGAREG( int port, char index )
{
 char valor;

  switch ( port )
   {
      case ATTRC:
	      inportb(STATUS_P); 
	      outportb(ATTRC, index);
              valor = inportb(ATTRC+1);
              outportb( ATTRC, 0x20 );
	      break;

      case VGASEQ: 
      case GRAPHC:
      case CRTC:
	      outportb(port, index);    
	      valor = inportb(port+1);
	      break;

      default:
	      valor = inportb(port);
             break;
   } 

return(valor);
}

/*========================================================================
   WriteVGAREG( controlador, index, valor );

   Escribe <valor> en el registro especificado, modificndolo.
   Controla los posibles casos comentados en la revista.
 ========================================================================*/
void WriteVGAREG( int port, char index, char valor )
{
  switch (port)
    {
      case ATTRC:
	   inportb(STATUS_P);
	   outportb(ATTRC, index);
	   outportb(ATTRC, valor );
           outportb( ATTRC, 0x20 );
	   break;

      case VGASEQ:
	   if (index == 1)
          {
             outport(VGASEQ, 0x0100);
	     outport(VGASEQ, valor<<8|1);
	     outport(VGASEQ, 0x0300);
	     break;
	    };
      case GRAPHC:
      case CRTC:
	   outport( port, index|valor<<8);
	   break;

      default:
	   outportb(port, valor);
	   break;
    } 
}

/*========================================================================
   UnProtectCRTC( void );

   Desprotege el CRTC contra escritura para opder modificar sus registros.
   Limpia el bit 7 del registro 11h del CRTC. (=1->protegido, =0->desprot.)
 ========================================================================*/
void UnProtectCRTC( void )
{
 int valor;
 outportb( CRTC, 0x11 );
 valor = inportb( CRTC+1 );
 valor = valor & 0x7F;
 outportb( CRTC, 0x11 );
 outportb( CRTC+1, valor);
}

/*========================================================================
   xxxwrite() y xxxread();

   Funciones directas a cada controlador comentadas en el artculo. Son.
   ms rpidas que WriteVGAREG y ReadVGAREG() si sabemos a qu registro
   concreto queremos acceder y es necesaria velocidad para ello.
 ========================================================================*/
char ATTRCread( char index )
{
char valor;
 inportb ( STATUS_P );
 outportb( ATTRC, index );
 valor = inportb( ATTRC+1 );
 outportb( ATTRC, 0x20 );
 return(valor);
}

/*=====================================================================*/
void ATTRCwrite( char index, char valor )
{
 inportb (STATUS_P);
 outportb( ATTRC, index );
 outportb( ATTRC, valor );

 outportb( ATTRC, 0x20 );
}

/*=====================================================================*/
char VGASEQread( char index )
{
 char valor;

 outportb( VGASEQ, index );
 valor = inportb( VGASEQ+1 );
 return(valor);
}

/*=====================================================================*/
void VGASEQwrite( char index, char valor )
{
 if ( index == 1 )
 {
  outport( VGASEQ, 0x0100 );
  outport( VGASEQ, valor<<8|1 );
  outport( VGASEQ, 0x0300 );
 }
 
 else
 {
  outportb( VGASEQ, index );
  outportb( VGASEQ+1, valor );
 }
}

/*=====================================================================*/
char CRTCread( char index )
{
 char valor;

 outportb( CRTC, index );
 valor = inportb( CRTC+1 );
 return(valor);
}

/*=====================================================================*/
void CRTCwrite( char index, char valor )
{
 outportb( CRTC, index );
 outportb( CRTC+1, valor );
}

/*=====================================================================*/
char GRAPHCread( char index )
{
 char valor;
 
 outportb( GRAPHC, index );
 valor = inportb( GRAPHC+1 );
 return(valor);
}

/*=====================================================================*/
void GRAPHCwrite( char index, char valor )
{
 outportb( GRAPHC, index );
 outportb( GRAPHC+1, valor );
}


/*========================================================================*/
/*---------------------- End Of File -------------------------------------*/
/*========================================================================*/
